#define com_maroontress_libcgi_util_ArraySet_IMPORT

#include <stdlib.h>
#include <com/maroontress/libcgi/util/ArraySet.h>

struct ArraySet {
    void **data;
    size_t size;
    size_t capacity;
};

struct ArraySet *
ArraySet_create(void)
{
    struct ArraySet *as;

    if ((as = (struct ArraySet *)malloc(sizeof(*as))) == NULL)
	return NULL;
    as->data = NULL;
    as->size = 0;
    as->capacity = 0;
    return as;
}

void
ArraySet_free(struct ArraySet *as)
{
    if (as == NULL)
	return;
    free(as->data);
    free(as);
}

unsigned int
ArraySet_getSize(struct ArraySet *as)
{
    return as->size;
}

void *
ArraySet_get(struct ArraySet *as, unsigned int k)
{
    if (k >= as->size)
	return NULL;
    return as->data[k];
}

int
ArraySet_add(struct ArraySet *as, void *e)
{
    static const size_t DELTA = 4;
    struct ArraySet next;

    next.size = as->size + 1;
    if (as->capacity < next.size) {
        next.capacity = next.size + DELTA;
        if ((next.data = realloc(as->data,
				 next.capacity * sizeof(e))) == NULL) {
            return -1;
        }
        as->data = next.data;
        as->capacity = next.capacity;
    }
    as->data[as->size] = e;
    as->size = next.size;
    return 0;
}

int
ArraySet_remove(struct ArraySet *as, void *e)
{
    unsigned int k;

    if (as->size == 0)
	return -1;
    for (k = 0; k < as->size && as->data[k] != e; ++k)
	;
    if (k == as->size)
	return -1;
    if (--(as->size) == 0) {
	free(as->data);
	as->data = NULL;
	as->capacity = 0;
	return 0;
    }
    as->data[k] = as->data[as->size];
    return 0;
}
